@@ -14,6 +14,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import collections
 import datetime
 
 from novaclient import exceptions as nova_exceptions
@@ -225,9 +226,9 @@ class PhysicalHostPlugin(base.BasePlugin, nova.NovaClientWrapper):
                           interval_end):
         """Heal reservations which suffer from resource failures.
 
-        :param: failed_resources: a list of failed hosts.
-        :param: interval_begin: start date of the period to heal.
-        :param: interval_end: end date of the period to heal.
+        :param failed_resources: a list of failed hosts.
+        :param interval_begin: start date of the period to heal.
+        :param interval_end: end date of the period to heal.
         :return: a dictionary of {reservation id: flags to update}
                  e.g. {'de27786d-bd96-46bb-8363-19c13b2c6657':
                        {'missing_resources': True}}
@@ -263,7 +264,7 @@ class PhysicalHostPlugin(base.BasePlugin, nova.NovaClientWrapper):
     def _reallocate(self, allocation):
         """Allocate an alternative host.
 
-        :param: allocation: allocation to change.
+        :param allocation: allocation to change.
         :return: True if an alternative host was successfully allocated.
         """
         reservation = db_api.reservation_get(allocation['reservation_id'])
@@ -541,21 +542,12 @@ class PhysicalHostPlugin(base.BasePlugin, nova.NovaClientWrapper):
 
         # To reduce overhead, this method only executes one query
         # to get the allocation information
-        allocs = db_utils.get_reservation_allocations_by_host_ids(
+        rsv_lease_host = db_utils.get_reservation_allocations_by_host_ids(
             hosts, start, end, lease_id, reservation_id)
 
-        hosts_allocs = {}
-        for reservation, alloc in allocs:
-            if alloc['compute_host_id'] in hosts_allocs:
-                hosts_allocs[alloc['compute_host_id']].append({
-                    'lease_id': reservation['lease_id'],
-                    'id': reservation['id']
-                })
-            else:
-                hosts_allocs[alloc['compute_host_id']] = [{
-                    'lease_id': reservation['lease_id'],
-                    'id': reservation['id']
-                }]
+        hosts_allocs = collections.defaultdict(list)
+        for rsv, lease, host in rsv_lease_host:
+            hosts_allocs[host].append({'lease_id': lease, 'id': rsv})
         return hosts_allocs
 
     def _matching_hosts(self, hypervisor_properties, resource_properties,
@@ -613,13 +605,8 @@ class PhysicalHostPlugin(base.BasePlugin, nova.NovaClientWrapper):
         return param
 
     def _check_params(self, values):
-        min_hosts = self._convert_int_param(values.get('min'), 'min')
-        max_hosts = self._convert_int_param(values.get('max'), 'max')
-
-        if 0 <= min_hosts and min_hosts <= max_hosts:
-            values['count_range'] = str(min_hosts) + '-' + str(max_hosts)
-        else:
-            raise manager_ex.InvalidRange()
+        self._validate_min_max_range(values, values.get('min'),
+                                     values.get('max'))
 
         if 'hypervisor_properties' not in values:
             raise manager_ex.MissingParameter(param='hypervisor_properties')
@@ -631,14 +618,23 @@ class PhysicalHostPlugin(base.BasePlugin, nova.NovaClientWrapper):
         if values['before_end'] not in before_end_options:
             raise manager_ex.MalformedParameter(param='before_end')
 
+    def _validate_min_max_range(self, values, min_hosts, max_hosts):
+        self._convert_int_param(min_hosts, 'min')
+        self._convert_int_param(max_hosts, 'max')
+        if min_hosts <= 0 or max_hosts <= 0:
+            raise manager_ex.MalformedParameter(
+                param='min and max (must be greater than or equal to 1)')
+        if max_hosts < min_hosts:
+            raise manager_ex.InvalidRange()
+        values['count_range'] = str(min_hosts) + '-' + str(max_hosts)
+
     def _update_allocations(self, dates_before, dates_after, reservation_id,
                             reservation_status, host_reservation, values):
-        min_hosts = self._convert_int_param(values.get(
-            'min', host_reservation['count_range'].split('-')[0]), 'min')
-        max_hosts = self._convert_int_param(values.get(
-            'max', host_reservation['count_range'].split('-')[1]), 'max')
-        if min_hosts < 0 or max_hosts < min_hosts:
-            raise manager_ex.InvalidRange()
+        min_hosts = values.get('min', int(
+            host_reservation['count_range'].split('-')[0]))
+        max_hosts = values.get(
+            'max', int(host_reservation['count_range'].split('-')[1]))
+        self._validate_min_max_range(values, min_hosts, max_hosts)
         hypervisor_properties = values.get(
             'hypervisor_properties',
             host_reservation['hypervisor_properties'])
@@ -875,7 +871,7 @@ class PhysicalHostMonitorPlugin(base.BaseMonitorPlugin,
     def _handle_failures(self, failed_hosts):
         """Handle resource failures.
 
-        :param: failed_hosts: a list of failed hosts.
+        :param failed_hosts: a list of failed hosts.
         :return: a dictionary of {reservation id: flags to update}
                  e.g. {'de27786d-bd96-46bb-8363-19c13b2c6657':
                  {'missing_resources': True}}
