@@ -35,15 +35,18 @@ class TestPlacementClient(tests.TestCase):
         self.cfg.config(os_auth_port='8080')
         self.cfg.config(os_auth_prefix='identity')
         self.cfg.config(os_auth_version='v3')
+        self.cfg.config(os_region_name='region_foo')
         self.client = placement.BlazarPlacementClient()
 
     def test_client_auth_url(self):
+        client = self.client._create_client()
         self.assertEqual("http://foofoo:8080/identity/v3",
-                         self.client._client.session.auth.auth_url)
+                         client.session.auth.auth_url)
 
     def _add_default_kwargs(self, kwargs):
         kwargs['endpoint_filter'] = {'service_type': 'placement',
-                                     'interface': 'public'}
+                                     'interface': 'public',
+                                     'region_name': 'region_foo'}
         kwargs['headers'] = {'accept': 'application/json'}
         kwargs['microversion'] = PLACEMENT_MICROVERSION
         kwargs['raise_exc'] = False
@@ -574,6 +577,25 @@ class TestPlacementClient(tests.TestCase):
                                           json=expected_data)
         self.assertEqual(mock_put_json, result)
 
+        kss_req.reset_mock()
+
+        # Test retrying on 409 conflict
+        mock_json_data = {
+            "errors": [
+                {"status": 409,
+                 "code": "placement.concurrent_update",
+                 "title": "Conflict"}
+            ]
+        }
+
+        kss_req.return_value = fake_requests.FakeResponse(
+            409, content=jsonutils.dump_as_bytes(mock_json_data))
+        self.assertRaises(
+            exceptions.InventoryConflict,
+            self.client.update_reservation_inventory, host_name, 'add', 3)
+        self.assertEqual(5, kss_req.call_count)
+        kss_req.reset_mock()
+
     @mock.patch('blazar.utils.openstack.placement.'
                 'BlazarPlacementClient.get_resource_provider')
     @mock.patch('keystoneauth1.session.Session.request')
