@@ -53,7 +53,10 @@ nova_opts = [
     cfg.StrOpt('blazar_owner',
                default='blazar:owner',
                deprecated_group=oshosts.RESOURCE_TYPE,
-               help='Aggregate metadata key for knowing owner project_id')
+               help='Aggregate metadata key for knowing owner project_id'),
+    cfg.BoolOpt('az_aware',
+                default=True,
+                help='A flag to store original availability zone')
 ]
 
 
@@ -120,13 +123,17 @@ class BlazarNovaClient(object):
             auth_token = auth_token or ctx.auth_token
             endpoint_override = endpoint_override or \
                 base.url_for(ctx.service_catalog,
-                             CONF.nova.compute_service)
-            auth_url = base.url_for(ctx.service_catalog, CONF.identity_service)
+                             CONF.nova.compute_service,
+                             os_region_name=CONF.os_region_name)
+            auth_url = base.url_for(ctx.service_catalog, CONF.identity_service,
+                                    os_region_name=CONF.os_region_name)
+            kwargs.setdefault('global_request_id', ctx.global_request_id)
 
         if auth_url is None:
-            auth_url = "%s://%s:%s/v3" % (CONF.os_auth_protocol,
-                                          CONF.os_auth_host,
-                                          CONF.os_auth_port)
+            auth_url = "%s://%s:%s/%s" % (CONF.os_auth_protocol,
+                                          base.get_os_auth_host(CONF),
+                                          CONF.os_auth_port,
+                                          CONF.os_auth_prefix)
 
         if username:
             kwargs.setdefault('username', username)
@@ -189,9 +196,7 @@ class NovaClientWrapper(object):
 
     @property
     def nova(self):
-        ctx = context.current()
-        nova = BlazarNovaClient(ctx=ctx,
-                                username=self.username,
+        nova = BlazarNovaClient(username=self.username,
                                 password=self.password,
                                 user_domain_name=self.user_domain_name,
                                 project_name=self.project_name,
@@ -206,7 +211,7 @@ class ReservationPool(NovaClientWrapper):
             password=CONF.os_admin_password,
             user_domain_name=CONF.os_admin_user_domain_name,
             project_name=CONF.os_admin_project_name,
-            project_domain_name=CONF.os_admin_user_domain_name)
+            project_domain_name=CONF.os_admin_project_domain_name)
 
         self.config = CONF.nova
         self.freepool_name = self.config.aggregate_freepool_name
@@ -254,8 +259,8 @@ class ReservationPool(NovaClientWrapper):
 
         name = name or self._generate_aggregate_name()
 
-        LOG.debug('Creating pool aggregate: %s with Availability Zone %s'
-                  % (name, az))
+        LOG.debug('Creating pool aggregate: %(name)s with Availability Zone '
+                  '%(az)s', {'name': name, 'az': az})
         agg = self.nova.aggregates.create(name, az)
 
         try:
@@ -263,7 +268,7 @@ class ReservationPool(NovaClientWrapper):
             project_id = ctx.project_id
         except RuntimeError:
             e = manager_exceptions.ProjectIdNotFound()
-            LOG.error(e.message)
+            LOG.error(str(e))
             raise e
 
         if metadata:
@@ -295,8 +300,8 @@ class ReservationPool(NovaClientWrapper):
         except manager_exceptions.AggregateNotFound:
             raise manager_exceptions.NoFreePool()
         for host in hosts:
-            LOG.debug("Removing host '%s' from aggregate "
-                      "'%s')" % (host, agg.id))
+            LOG.debug("Removing host '%(host)s' from aggregate '%(id)s')",
+                      {'host': host, 'id': agg.id})
             self.nova.aggregates.remove_host(agg.id, host)
 
             if freepool_agg.id != agg.id and host not in freepool_agg.hosts:
@@ -323,20 +328,24 @@ class ReservationPool(NovaClientWrapper):
         except manager_exceptions.AggregateNotFound:
             return []
 
-    def add_computehost(self, pool, host, stay_in=False):
-        """Add a compute host to an aggregate.
+    def add_computehost(self, pool, hosts, stay_in=False):
+        """Add compute host(s) to an aggregate.
 
-        The `host` must exist otherwise raise an error
-        and the `host` must be in the freepool.
+        Each host must exist and be in the freepool, otherwise raise an error.
 
         :param pool: Name or UUID of the pool to rattach the host
-        :param host: Name (not UUID) of the host to associate
-        :type host: str
+        :param hosts: Names (not UUID) of hosts to associate
+        :type host: str or list of str
 
         Return the related aggregate.
         Raise an aggregate exception if something wrong.
         """
 
+        if not isinstance(hosts, list):
+            hosts = [hosts]
+
+        added_hosts = []
+        removed_hosts = []
         agg = self.get_aggregate_from_name_or_id(pool)
 
         try:
@@ -344,25 +353,44 @@ class ReservationPool(NovaClientWrapper):
         except manager_exceptions.AggregateNotFound:
             raise manager_exceptions.NoFreePool()
 
-        if freepool_agg.id != agg.id and not stay_in:
-            if host not in freepool_agg.hosts:
-                raise manager_exceptions.HostNotInFreePool(
-                    host=host, freepool_name=freepool_agg.name)
-            LOG.info("removing host '%s' "
-                     "from aggregate freepool %s" % (host, freepool_agg.name))
-            try:
-                self.remove_computehost(freepool_agg.id, host)
-            except nova_exception.NotFound:
-                raise manager_exceptions.HostNotFound(host=host)
-
-        LOG.info("adding host '%s' to aggregate %s" % (host, agg.id))
         try:
-            return self.nova.aggregates.add_host(agg.id, host)
-        except nova_exception.NotFound:
-            raise manager_exceptions.HostNotFound(host=host)
-        except nova_exception.Conflict:
-            raise manager_exceptions.AggregateAlreadyHasHost(pool=pool,
-                                                             host=host)
+            for host in hosts:
+                if freepool_agg.id != agg.id and not stay_in:
+                    if host not in freepool_agg.hosts:
+                        raise manager_exceptions.HostNotInFreePool(
+                            host=host, freepool_name=freepool_agg.name)
+                    LOG.info("removing host '%(host)s' from freepool "
+                             "aggregate %(name)s",
+                             {'host': host, 'name': freepool_agg.name})
+                    try:
+                        self.remove_computehost(freepool_agg.id, host)
+                        removed_hosts.append(host)
+                    except nova_exception.NotFound:
+                        raise manager_exceptions.HostNotFound(host=host)
+
+                LOG.info("adding host '%(host)s' to aggregate %(id)s",
+                         {'host': host, 'id': agg.id})
+                try:
+                    self.nova.aggregates.add_host(agg.id, host)
+                    added_hosts.append(host)
+                except nova_exception.NotFound:
+                    raise manager_exceptions.HostNotFound(host=host)
+                except nova_exception.Conflict as e:
+                    raise manager_exceptions.AggregateAlreadyHasHost(
+                        pool=pool, host=host, nova_exception=str(e))
+        except Exception as e:
+            if added_hosts:
+                LOG.warn('Removing hosts added to aggregate %s: %s',
+                         agg.id, added_hosts)
+                for host in added_hosts:
+                    self.nova.aggregates.remove_host(agg.id, host)
+            if removed_hosts:
+                LOG.warn('Adding hosts back to freepool: %s', removed_hosts)
+                for host in removed_hosts:
+                    self.nova.aggregates.add_host(freepool_agg.name, host)
+            raise e
+
+        return self.get_aggregate_from_name_or_id(pool)
 
     def remove_all_computehosts(self, pool):
         """Remove all compute hosts attached to an aggregate."""
@@ -395,7 +423,7 @@ class ReservationPool(NovaClientWrapper):
                 self.nova.aggregates.remove_host(agg.id, host)
             except nova_exception.ClientException:
                 hosts_failing_to_remove.append(host)
-            if freepool_agg.id != agg.id:
+            if freepool_agg.id != agg.id and host not in freepool_agg.hosts:
                 # NOTE(sbauza) : We don't want to put again the host in
                 # freepool if the requested pool is the freepool...
                 try:
@@ -436,29 +464,48 @@ class NovaInventory(NovaClientWrapper):
     def get_host_details(self, host):
         """Get Nova capabilities of a single host
 
-        :param host: UUID or name of nova-compute host
+        :param host: UUID, ID or name of nova-compute host
         :return: Dict of capabilities or raise HostNotFound
         """
         try:
+            # NOTE(tetsuro): Only id (microversion < 2.53) or uuid
+            # (microversion >= 2.53) is acceptable for the
+            # `novaclient.hypervisors.get` argument. The invalid arguments
+            # result in NotFound exception for microversion < 2.53 and
+            # BadRequest exception for microversion >= 2.53
             hypervisor = self.nova.hypervisors.get(host)
-        except nova_exception.NotFound:
+        except (nova_exception.NotFound, nova_exception.BadRequest):
+            # Name (not id or uuid) is given for the `host` parameter.
             try:
                 hypervisors_list = self.nova.hypervisors.search(host)
             except nova_exception.NotFound:
                 raise manager_exceptions.HostNotFound(host=host)
             if len(hypervisors_list) > 1:
-                raise manager_exceptions.MultipleHostsFound(host)
+                raise manager_exceptions.MultipleHostsFound(host=host)
             else:
                 hypervisor_id = hypervisors_list[0].id
                 # NOTE(sbauza): No need to catch the exception as we're sure
                 #  that the hypervisor exists
                 hypervisor = self.nova.hypervisors.get(hypervisor_id)
+
+        az_name = ''
+        if CONF.nova.az_aware:
+            host_name = hypervisor.hypervisor_hostname
+            for zone in self.nova.availability_zones.list(detailed=True):
+                if (host_name in zone.hosts
+                   and 'nova-compute' in zone.hosts[host_name]):
+                    az_name = zone.zoneName
+
         try:
+            # NOTE(tetsuro): compute API microversion 2.28 changes cpu_info
+            # from string to object
+            cpu_info = str(hypervisor.cpu_info)
             return {'id': hypervisor.id,
+                    'availability_zone': az_name,
                     'hypervisor_hostname': hypervisor.hypervisor_hostname,
                     'service_name': hypervisor.service['host'],
                     'vcpus': hypervisor.vcpus,
-                    'cpu_info': hypervisor.cpu_info,
+                    'cpu_info': cpu_info,
                     'hypervisor_type': hypervisor.hypervisor_type,
                     'hypervisor_version': hypervisor.hypervisor_version,
                     'memory_mb': hypervisor.memory_mb,
@@ -477,7 +524,7 @@ class NovaInventory(NovaClientWrapper):
         except nova_exception.NotFound:
             raise manager_exceptions.HostNotFound(host=host)
         if len(hypervisors_list) > 1:
-            raise manager_exceptions.MultipleHostsFound(host)
+            raise manager_exceptions.MultipleHostsFound(host=host)
         else:
             try:
                 return hypervisors_list[0].servers
